/* winthread.cpp
 * encapsulates the Windows thread
 * Copyright (C) 2003 Michel Leunen
 * http://www.leunen.com/
 * michel@leunen.com
 *
 * This software is freeware and is provided at no charge to the user.
 * This software is provided 'as-is', without any express or implied warranty.
 * To the maximum extent permitted by applicable law will the author NOT be held
 * liable for any damages whatsoever arising from the use of this software.
 *
 * Permission is granted to anyone to use this software for any purpose, including
 * commercial applications, and to freely redistribute it, with the following
 * restrictions:
 *
 * 1. If you use this software in a product, an acknowledgment in the product
 *    documentation would be appreciated.
 *
 * 2. If you use this software in a commercial application, please send me an email
 *	  detailing the application.
 *
 * 3. This notice may not be removed or altered from any source distribution.
 *
 * 4. It may be distributed by any means, provided that the original files
 *    as supplied by the author remain intact and no charge is made other than
 *    for reasonable distribution costs. If you are a vendor, if you want to
 *    include this software in a compilation or in a magazine cover disk
 *    you HAVE TO CONTACT ME to obtain my authorization before distribution.
 */

#include "winthread.h"

mlLib::WinThread::WinThread():
		handle_(0),id_(0),suspended_(true),Terminated(false)
{

}

mlLib::WinThread::~WinThread()
{
	if(handle_)CloseHandle(handle_);
}

bool mlLib::WinThread::Create()
{
	handle_=CreateThread(NULL,0,ThreadFunc,this,CREATE_SUSPENDED,&id_);
	if(handle_)return true;
	else return false;
}

void mlLib::WinThread::Resume()
{
  if(suspended_)
  {
    Lock protect(cs_);
    if(suspended_)
    {
      ResumeThread(handle_);
      suspended_=false;
    }
    }
}

void mlLib::WinThread::Suspend()
{
  if(!suspended_)
  {
    Lock protect(cs_);
    if(!suspended_)
    {
      SuspendThread(handle_);
      suspended_=true;
    }
  }
}

void mlLib::WinThread::Terminate()
{
  if(!Terminated)
  {
    Lock protect(cs_);
    if(!Terminated)Terminated=true;
    Resume();
  }
}

void mlLib::WinThread::WaitFor(DWORD delay)
{
  WaitForSingleObject(handle_,delay);
}

DWORD __stdcall mlLib::WinThread::ThreadFunc(void* arg)
{
  WinThread *ptr=reinterpret_cast<WinThread*>(arg);
  if(ptr)ptr->Execute();
  return 0;
}
